# import sys

# from PyQt6.QtWidgets import QApplication

# from starrail.gui.application import StarRailToolkitWindow


# def gui_entry():
#     app = QApplication(sys.argv)
#     _ = StarRailToolkitWindow()
#     sys.exit(app.exec())


# if __name__ == '__main__':
#     gui_entry()
