import starrail.utils.babelfish.dictionary as dictionary
from starrail.utils.babelfish.translate import translate

average_gacha_per_5_star = dictionary.average_gacha_per_5_star
gacha_report = dictionary.gacha_report
gacha_title = dictionary.gacha_title
history_of_5_stars = dictionary.history_of_5_stars
html_thead = dictionary.html_thead
markdown_thead = dictionary.markdown_thead

__all__ = ['translate']
